from __future__ import annotations

import numpy as np
import pytest

from cvx.bson.file import from_bson, read_bson, to_bson, write_bson


@pytest.mark.parametrize("shape", [(50, 50), (1000, 50), (50, 1000), (1000, 1000), (5000, 2000)])
def test_write(tmp_path, shape):
    """
    Test that a numpy array is written and read correctly

    Args:
        tmp_path: temporary path fixture
        shape: shape of the matrix
    """
    data = {"a": np.ones(shape)}
    write_bson(data=data, file=tmp_path / "maffay.bson")

    x = read_bson(file=tmp_path / "maffay.bson")

    assert set(x.keys()) == {"a"}
    np.allclose(x["a"], data["a"])


def test_vector(tmp_path):
    """
    Test that a vector is written and read correctly

    Args:
        tmp_path: temporary path fixture
    """
    data = {"a": np.array([1, 2, 3])}
    write_bson(data=data, file=tmp_path / "maffay.bson")

    x = read_bson(file=tmp_path / "maffay.bson")

    assert set(x.keys()) == {"a"}
    np.allclose(x["a"], data["a"])


@pytest.mark.parametrize("shape", [(50, 50), (1000, 50), (50, 1000), (1000, 1000), (5000, 2000)])
def test_without_file(shape):
    """
    Test that a matrix is written and read correctly

    Args:
        shape: shape of the matrix
    """
    data = {"a": np.ones(shape)}
    # Note that it takes already 40ms
    recovered = from_bson(to_bson(data))
    assert set(recovered.keys()) == {"a"}
    # it takes signifianctly longer with this line
    assert np.allclose(recovered["a"], data["a"])



import marimo

__generated_with = "0.9.27"
app = marimo.App()


@app.cell
def __(mo):
    mo.md(r"""# cvxbson""")
    return


@app.cell
def __():
    import numpy as np

    from cvx.bson import read_bson, write_bson
    from cvx.json import read_json, write_json

    return np, read_bson, read_json, write_bson, write_json


@app.cell
def __(mo):
    mo.md(r"""## Create a dictionary of numpy arrays""")
    return


@app.cell
def __(np):
    data = {"A": np.random.rand(50, 50), "B": np.random.rand(50)}
    return (data,)


@app.cell
def __(mo):
    mo.md(r"""## json""")
    return


@app.cell
def __(data, np, read_json, write_json):
    write_json("test.json", data)
    _recovered = dict(read_json("test.json"))
    assert np.allclose(data["A"], _recovered["A"])
    assert np.allclose(data["B"], _recovered["B"])
    return


@app.cell
def __(mo):
    mo.md(r"""## bson""")
    return


@app.cell
def __(data, np, read_bson, write_bson):
    write_bson("test.bson", data)
    _recovered = dict(read_bson("test.bson"))
    assert np.allclose(data["A"], _recovered["A"])
    assert np.allclose(data["B"], _recovered["B"])
    return


@app.cell
def __():
    import marimo as mo

    return (mo,)


if __name__ == "__main__":
    app.run()



import numpy as np
import pytest

from cvx.json import read_json, write_json


@pytest.mark.parametrize("shape", [(50, 50), (1000, 50), (50, 1000), (1000, 1000), (5000, 2000)])
def test_read_and_write_json(tmp_path, shape):
    """
    Test that a numpy array is written and read correctly

    Args:
        tmp_path: temporary path fixture
        shape: shape of the matrix
    """
    data = {"a": np.random.rand(*shape), "b": 3.0, "c": "test"}
    write_json(tmp_path / "test.json", data)

    recovered_data = dict(read_json(tmp_path / "test.json"))

    assert data["b"] == recovered_data["b"]
    assert data["c"] == recovered_data["c"]

    # check numpy arrays are equal
    np.testing.assert_array_equal(data["a"], recovered_data["a"])



from dataclasses import dataclass, field
from typing import Dict

import polars as pl

from cvx.bson.dataclass import Data


@dataclass(frozen=True)
class DataAPI(Data):
    # List of tables expected
    A: pl.DataFrame
    B: pl.DataFrame
    C: pl.DataFrame

    # Define a mutable value for the mapping from short name to full name
    tables: Dict[str, str] = field(default_factory=lambda: {"A": "AAA", "B": "BBB", "C": "CCC"})


if __name__ == "__main__":
    # We add this as a little demo
    import numpy as np

    data = DataAPI(
        A=pl.DataFrame(np.random.rand(10, 5)),
        B=pl.DataFrame(np.random.rand(20, 3)),
        C=pl.DataFrame(np.random.rand(50, 5)),
    )

    # We can access the tables
    print(data.tables)
    print(data.A)
    print(data.B)
    print(data.C)

    # convert all data into one bson file
    print(data.to_bson("data.bson"))

    data2 = DataAPI.from_bson("data.bson")
    print(data2)

    def strategy(api: DataAPI):
        # the strategy has no idea how the data is stored
        print(api.A)

    strategy(data2)



import numpy as np
import pandas as pd
import polars as pl
import pytest

from cvx.bson.file import from_bson, to_bson


@pytest.fixture()
def data():
    return {
        "frame": pd.DataFrame(data=np.random.rand(5, 2)),
        "numpy": np.random.rand(5, 2),
        "frame_with_time": pd.DataFrame(
            data=np.random.rand(2, 2),
            index=[pd.Timestamp("2020-01-01"), pd.Timestamp("2022-01-01")],
        ),
        "polars": pl.DataFrame(data=np.random.rand(10, 3)),
    }


def assert_equal(obj1, obj2):
    assert type(obj1) is type(obj2)

    if isinstance(obj1, pd.DataFrame):
        pd.testing.assert_frame_equal(obj1, obj2)

    if isinstance(obj1, np.ndarray):
        np.testing.assert_array_equal(obj1, obj2)

    if isinstance(obj1, pl.DataFrame):
        assert obj1.equals(obj2)


def test_roundtrip(data):
    """
    Testing the roundtrip

    Args:
        data: Fixture exposing a dictionary of data
    """
    reproduced = from_bson(to_bson(data))
    for key, value in reproduced.items():
        assert_equal(value, data[key])


def test_file(data, tmp_path):
    with open(file=tmp_path / "xxx.bson", mode="wb") as bson_file:
        bson_file.write(to_bson(data))

    with open(file=tmp_path / "xxx.bson", mode="rb") as bson_file:
        reproduced = from_bson(bson_file.read())

    for key, value in reproduced.items():
        assert_equal(reproduced[key], data[key])



import numpy as np
import pandas as pd
import polars as pl

from cvx.bson.io import decode, encode


def test_pl():
    frame = pl.DataFrame(data=np.random.rand(10, 5))
    b = decode(encode(frame))
    assert frame.equals(b)


def test_pd():
    frame = pd.DataFrame(data=np.random.rand(10, 5))
    b = decode(encode(frame))
    pd.testing.assert_frame_equal(b, frame)


def test_np():
    matrix = np.random.rand(10, 5)
    b = decode(encode(matrix))
    np.testing.assert_array_equal(matrix, b)
